// SQLdb01.cpp
//
// Test the various MPH ODBC/SQL class wrappers in a console 
// application.
//
// Revision Log
//
// Date       Who SAR     Notes
// ========== === ======= =====================================
// 10/07/2001 mph         Initial coding
// 
//

#include "stdafx.h"
#include <iostream>
#include <iomanip>
#include <string>
using namespace std;

#include "Classes\SQLDatabase.h"
#include "Classes\SQLRecordset.h"
#include "Classes\SQLInsert.h"
#include "Classes\SQLUpdate.h"
#include "Classes\SQLDelete.h"

int main(int argc, char* argv[])
{
   try
   {
      CSQLDatabase db;
      if ( ! db.Open( "MPHSample" ) ) 
      {
         cout << "couldn't open data source name MPHSample" << endl;
         return 0;
      }

      cout << "good db open" << endl;
      cout << "supports transactions: " << (char*)((db.SupportsTransactions())?"yes":"no") << endl;
      db.SetAutoCommit( true );
      cout << "try getting some data..." << endl;

      ////////////////////////////////////////////////////
      //
      // Start by printing some records from the database
      //
      //
      { // so the recordset is destroyed
         CSQLRecordset rs( db );
         rs << "select provider, address1, city, state,"
            " zip from provider";
         rs.SetWhere( "provider < 'B'" ); // lt 1 screen full
         rs.SQLExecDirect();
         while ( rs.SQLFetch() )
         {
            int n = 1; // order is crucial; hence, the ctr
            string sProvider = rs.SQLGetData( n++ );
            string sAddress1 = rs.SQLGetData( n++ );
            string sCity = rs.SQLGetData( n++ );
            string sState = rs.SQLGetData( n++ );
            string sZip = rs.SQLGetData( n++ );

            cout  << left
                  << setw(22) << sProvider
                  << sAddress1 << ", "
                  << sCity << ", "
                  << sState << "  "
                  << sZip
                  << endl;
         }
      }

      ////////////////////////////////////////////////////
      //
      // add a record
      //
      //
      cout << "insert Josh Angler's record..." << endl;

      {
         CSQLInsert s( "provider" );
         s.SetColumn( "provider", "Angler, Josh" );
         // note that the column name's case is ignored
         // by the odbc translator; however, the data
         // goes in exactly as you typed it.
         s.SetColumn( "aDDRess1", "123 Anystreet" ); 
         s.SetColumn( "City", "Spokane" );
         s.SetColumn( "state", "WA" );
         s.SetColumn( "zip", "99204" );
         s.SetColumn( "dob", true, 12, 13, 1955 );
         s.SetColumn( "referrals", 0 );
         db.ExecuteSQL( s.GetStatement() );
      }

      ////////////////////////////////////////////////////
      //
      // update a record
      //
      //
      {
         cout << "update Josh Angler's record..." << endl;
         CSQLUpdate s( "provider" );
         // where clause necessary for update
         s.SetWhere( "provider", "Angler, Josh" );
         s.SetColumn( "address1", "2903 West Pacific Ave." );
         db.ExecuteSQL( s.GetStatement() );
      }

      ////////////////////////////////////////////////////
      //
      // Print records after the changes
      //
      //
      {
         cout << endl << endl << "Print records after changes..." << endl;
         CSQLRecordset rs( db );
         rs << "select provider, address1, city, state,"
            " zip, dob from provider";
         rs.SetWhere( "provider < 'B'" ); // lt 1 screen full
         rs.SQLExecDirect();
         while ( rs.SQLFetch() )
         {
            int n = 1; // order is crucial; hence, the ctr
            string sProvider = rs.SQLGetData( n++ );
            string sAddress1 = rs.SQLGetData( n++ );
            string sCity = rs.SQLGetData( n++ );
            string sState = rs.SQLGetData( n++ );
            string sZip = rs.SQLGetData( n++ );
            string sDOB = rs.SQLGetData( n++ );

            cout  << left
                  << setw(22) << sProvider
                  << sAddress1 << ", "
                  << sCity << ", "
                  << sState << "  "
                  << sZip << " "
                  << sDOB
                  << endl;
         }
      }

      ////////////////////////////////////////////////////
      //
      // delete a record
      //
      //
      {
         cout << "delete Josh Angler's record..." << endl;
         CSQLDelete s( "provider" );
         // where clause necessary for delete
         s.SetWhere( "provider", "Angler, Josh" );
         cout << s.GetStatement() << endl;
         db.ExecuteSQL( s.GetStatement() );
      }

      ////////////////////////////////////////////////////
      //
      // close and reopen the database object
      //
      //
      cout << "close the database connection" << endl;;
      db.Close();
      cout << "open the database connection again" << endl;
      if ( ! db.Open( "MPHSample" ) ) 
      {
         cout << "couldn't open data source name MPHSample a second time on the same object" << endl;
         return 0;
      }
      cout << "good 2nd db open using same object" << endl;
   }
   catch ( CSQLException* e )
   {
      cout << endl << "Error on delete" << endl;
      cout << e->m_strError << endl;
   }
   catch (...)
   {
      cout << "Unknown exception: It's likely that you closed the db object while a recordset was still active." << endl;
   }

	return 0;
}
